@@ -1,9 +1,13 @@
 # coding=utf-8
+from typing import Generator
 import requests
 import re
 import json
 from lxml import etree
 from datetime import datetime, timedelta
+from functools import wraps
+
+from requests.exceptions import HTTPError
 
 xgxt_login = 学工系统登录 = "https://cas.dgut.edu.cn/home/Oauth/getToken/appid/xgxtt.html"
 illness_login = 疫情防控系统登录 = "https://cas.dgut.edu.cn/home/Oauth/getToken/appid/illnessProtectionHome/state/home.html"
@@ -13,8 +17,12 @@ jwxt_login = 教务系统登录 = "https://cas.dgut.edu.cn/home/Oauth/getToken/a
 def decorator_signin(url: str):
     '''
     定义一个登录装饰器，url是系统的登录url
+    :param url: str
+    :return: function
     '''
     def decorator(func):
+        wraps(func)
+
         def wrapper(self, *args, **kargs):
             self.signin(url)
             return func(self, *args, **kargs)
@@ -22,59 +30,62 @@ def decorator_signin(url: str):
     return decorator
 
 
+def detect_type(detection_name, detection, type_: type):
+    '''
+    类型检测函数，如果符合类型返回该变量，如果不符合类型，则抛出TypeError
+    :param detection: type
+    '''
+    if not isinstance(detection, type_):
+        raise TypeError(
+            f"[变量类型错误] 变量「{detection_name} = {detection}」不能是{type(detection)}类型，请修改为{type_}类型")
+    return detection
+
+
 class AuthError(Exception):
     '''
     认证错误类
     '''
 
-    def __init__(self, reason):
+    def __init__(self, reason) -> None:
         self.reason = reason
 
-    def __str__(self):
-        print(f"认证失败,失败原因:{self.reason}")
+    def __str__(self) -> None:
+        print(f"[认证失败] 失败原因：{self.reason}")
 
 
 class dgutUser(object):
     '''
     莞工用户类
-
-    username : 中央认证账号用户名
-
-    password : 中央认证账号密码
     '''
 
-    def __init__(self, username: str, password: str):
+    def __init__(self, username: str, password: str) -> None:
         '''
-        构造函数__init__(self, username, password)
-
-        username : 中央认证账号用户名
-
-        password : 中央认证账号密码
+        :param username(中央认证账号): str
+        :param password(中央认证密码): str
+        :return: None
         '''
-        self.username = str(username)
-        self.__password = str(password)
+        self.username = detect_type("username", username, str)
+        self.__password = detect_type("password", password, str)
 
         # 创建一个会话
         self.session = requests.Session()
         # 设置请求超时时间
-        self.timeout = 20
+        self.timeout = 30
 
         self.session.headers = {
             'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.141 Safari/537.36',
         }
 
-    def signin(self, login_url: str):
+    def signin(self, login_url: str) -> requests.Response:
         '''
         登录函数
-
-        signin(self, login_url)
-        login_url : 系统登录url
+        :param login_url(系统登录url): str
+        :return: requests.Response
         '''
         # 检查url
-        if not isinstance(login_url, str):
-            raise TypeError("login_url参数应为str类型")
-        if not re.match(r"^https?://(.*?\.){1,}.*", str(login_url)):
-            raise ValueError(f"url<{login_url}>的格式错误")
+        detect_type("login_url", login_url, str)
+        if not re.match(r"^https?://[^\.]{1,}(\.[^\.]{1,})*", str(login_url)):
+            raise ValueError(f"[格式错误] 变量「url = {login_url}」的格式错误")
         # 获取登录token
         response = self.session.get(
             login_url, timeout=self.timeout)
@@ -90,7 +101,9 @@ class dgutUser(object):
             }
             response = self.session.post(
                 login_url, data=data, timeout=self.timeout)
-            result = json.loads(response.text)
+            if not response.status_code == 200:
+                raise HTTPError(f"[HTTP响应错误] HTTP code {response.status_code}")
+            result = response.json()
             if result['code'] == 1:
                 # 登录成功
                 # 认证
@@ -115,13 +128,9 @@ class dgutUser(object):
 class dgutXgxt(dgutUser):
     '''
     莞工学工系统类
-
-    username : 中央认证账号用户名
-
-    password : 中央认证账号密码
     '''
 
-    def __init__(self, username: str, password: str):
+    def __init__(self, username: str, password: str) -> None:
         dgutUser.__init__(self, username, password)
         self.session.headers = {
             'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.141 Safari/537.36',
@@ -129,48 +138,38 @@ class dgutXgxt(dgutUser):
         }
 
     @decorator_signin(xgxt_login)
-    def get_workAssignment(self):
+    def get_workAssignment(self) -> list:
         '''
         获取考勤职位信息
+        :return: list
         '''
         # 请求考勤页职位信息
         response = self.session.get(
             "http://stu.dgut.edu.cn/student/partwork/attendance.jsp", timeout=self.timeout)
-        html = etree.HTML(response.text)
-        result = html.xpath('//div[@class="searchTitle"]')
-        works = result[0].xpath('./select/option[position()>1]')
-        workAssignment = []
-        if not len(works):
-            return list()
-        for item in works:
-            id_ = item.xpath('./@value')
-            name = item.xpath('./text()')
-            workAssignment.append((id_[0], name[0]))
-        return workAssignment
+        if not response.status_code == 200:
+            raise HTTPError(f"[HTTP响应错误] HTTP code {response.status_code}")
+        works = etree.HTML(response.text).xpath(
+            '//div[@class="searchTitle"]/select/option')
+        return [(item.xpath('./@value'), item.xpath('./text()')) for item in works[1:]]
 
     @decorator_signin(xgxt_login)
-    def attendance(self, flag: int, workAssignmentId: int = None):
+    def attendance(self, flag: int, workAssignmentId: str = None) -> dict:
         '''
         学工系统考勤
-
-        attendance(self, flag[, workAssignmentId])
-        flag : 1->签到 / 2->签退
-        workAssignmentId : 职位id，为None时自动获取当前的首个职位
+        :param flag: int: 1->签到 | 2->签退
+        :param workAssignmentId(职位id，为None时自动获取当前的首个职位): str
+        :return: dict
         '''
         if not flag in [1, 2]:
-            raise ValueError("参数flag只能是1或2")
-        if flag == 1:
-            action_name = 'beginWork'
-            s = '签到'
-        if flag == 2:
-            action_name = 'endWork'
-            s = '签退'
+            raise ValueError("[参数错误] 参数flag只能是1或2")
+        action_name, s = ('beginWork', '签到') if flag == 1 else (
+            'endWork', '签退')
 
         # 如果没有传递workAssignmentId，自动获取可考勤的第一个职位作为考勤职位
         if not workAssignmentId:
             workAssignment = self.get_workAssignment()
             if len(workAssignment) < 1:
-                raise ValueError("没有可考勤的职位")
+                raise ValueError("[参数错误] 没有可考勤的职位")
             workAssignmentId = workAssignment[0][0]
 
         # 请求考勤界面
@@ -208,36 +207,40 @@ class dgutXgxt(dgutUser):
 class dgutIllness(dgutUser):
     '''
     莞工疫情防控系统类
-
-    username : 中央认证账号用户名
-
-    password : 中央认证账号密码
     '''
 
-    def __init__(self, username: str, password: str):
+    def __init__(self, username: str, password: str) -> None:
         dgutUser.__init__(self, username, password)
         self.session.headers = {
             'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.141 Safari/537.36',
             'Host': 'yqfk.dgut.edu.cn',
         }
 
-    @decorator_signin(illness_login)
-    def report(self, longitude: float = 113.87651, latitude: float = 22.90701):
+    # @decorator_signin(illness_login)
+    def report(self, longitude: float = 113.87651, latitude: float = 22.90701) -> dict:
+        '''
+        疫情打卡
+        :param longitude(所在位置经度): float
+        :param latitude(所在位置纬度): float
+        :return: dict
+        '''
         # 1、获取access_token
         response = self.signin(illness_login)
         access_token = re.search(
             r'access_token=(.*)', response.url, re.S)
         if not access_token:
-            raise AuthError("获取access_token失败")
+            raise ValueError("[参数错误] 获取access_token失败")
         access_token = access_token.group(1)
         self.session.headers['authorization'] = 'Bearer ' + access_token
 
         # 2、获取并修改数据
         response = self.session.get(
             'https://yqfk.dgut.edu.cn/home/base_info/getBaseInfo')
-        if json.loads(response.text)['code'] != 200:
+        if not response.json()['code'] == 200:
             raise AuthError("获取个人基本信息失败")
-        data = json.loads(response.text)['info']
+        data = response.json()['info']
+        if "已连续打卡" in data.get("msg"):
+            return {"code": 400, "message": data.get("msg"), "info": []}
         pop_list = [
             'can_submit',
             'class_id',
@@ -265,27 +268,22 @@ class dgutIllness(dgutUser):
                 data.pop(key)
 
         # 获取GPS位置
-        response = self.session.get(
-            "https://yqfk.dgut.edu.cn/home/base_info/getGPSAddress", params={
-                'longitude': longitude,
-                'latitude': latitude})
+        self.session.get("https://yqfk.dgut.edu.cn/home/base_info/getGPSAddress", params={
+            'longitude': longitude,
+            'latitude': latitude})
 
         # 4、提交数据
         response = self.session.post(
             "https://yqfk.dgut.edu.cn/home/base_info/addBaseInfo", json=data)
-        return json.loads(response.text)
+        return response.json()
 
 
 class dgutJwxt(dgutUser):
     '''
     莞工教务系统类
-
-    username : 中央认证账号用户名
-
-    password : 中央认证账号密码
     '''
 
-    def __init__(self, username: str, password: str):
+    def __init__(self, username: str, password: str) -> None:
         dgutUser.__init__(self, username, password)
         self.session.headers = {
             'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.141 Safari/537.36',
@@ -293,29 +291,29 @@ class dgutJwxt(dgutUser):
         }
 
     @decorator_signin(jwxt_login)
-    def get_score(self, score_type: int = 1, course_type: int = 3, time_range: int = 3, **kwargs):
+    def get_score(self, score_type: int = 1, course_type: int = 3, time_range: int = 3, **kwargs) -> Generator:
         '''
         获取成绩信息
-
-        score_type: 成绩类型: 1=>原始成绩（默认） | 2=>有效成绩
-
-        course_type: 课程类型: 1=>主修 | 2=>辅修 | 3=>主修和辅修（默认）
-
-        time_range: 时间范围选择: 1=>入学以来 | 2=>学年 | 3=>学期（默认）
+        :param score_type(成绩类型): int: 1=>原始成绩（默认） | 2=>有效成绩
+        :param course_type(课程类型): int: 1=>主修 | 2=>辅修 | 3=>主修和辅修（默认）
+        :param time_range(时间范围选择): int: 1=>入学以来 | 2=>学年 | 3=>学期（默认）
+        :param kwargs: dict: xn(学年):int & xq(学期):int:1|2
+        :return: list
         '''
         # 字段合法检测
         if not score_type in [1, 2]:
-            raise ValueError(f"score_type取值只能是1|2，而你的取值为{score_type}")
+            raise ValueError(f"[参数错误] score_type取值只能是1|2，而你的取值为{score_type}")
         if not course_type in [1, 2, 3]:
-            raise ValueError(f"course_type取值只能是1|2|3，而你的取值为{course_type}")
+            raise ValueError(
+                f"[参数错误] course_type取值只能是1|2|3，而你的取值为{course_type}")
         if not time_range in [1, 2, 3]:
-            raise ValueError(f"time_range取值只能是1|2|3，而你的取值为{time_range}")
+            raise ValueError(f"[参数错误] time_range取值只能是1|2|3，而你的取值为{time_range}")
         data = {
             "sjxz": "sjxz" + str(time_range),  # 时间选择，1-入学以来，2-学年，3-学期
             # 原始有效，指原始成绩或有效成绩，yscj-原始成绩，yxcj-有效成绩
             "ysyx": "yscj" if score_type == 1 else "yxcj",
             "zx": 1 if not course_type == 2 else 0,  # 主修，1-选择，0-不选择
-            "fx": 1 if not course_type == 3 else 0,  # 辅修，1-选择，0-不选择
+            "fx": 1 if not course_type == 1 else 0,  # 辅修，1-选择，0-不选择
             # xn-xn1学年第(xq+1)学期
             "xn": "",
             "xn1": "",
@@ -335,7 +333,7 @@ class dgutJwxt(dgutUser):
 
             if not isinstance(xn, int) and 1970 <= xn <= 9999:
                 raise ValueError(
-                    f"xn的类型应是int且取值范围为[1970, 9999]，而不应该是{xn}: {type(xn)}")
+                    f"[参数错误] xn的类型应是int且取值范围为[1970, 9999]，而不应该是「{xn}: {type(xn)}」")
             data['xn'] = xn
             data['xn1'] = xn+1
 
@@ -346,16 +344,16 @@ class dgutJwxt(dgutUser):
                     xq = 1 if now.month >= 9 else 2
                 if not isinstance(xq, int) and xq in [1, 2]:
                     raise ValueError(
-                        f"xq的类型应是int且只能是1|2，而不应该是{xq}: {type(xq)}")
+                        f"[参数错误] xq的类型应是int且只能是1|2，而不应该是「{xq}: {type(xq)}」")
                 xq -= 1
                 data['xq'] = xq
 
         # 发送请求
-        responsse = self.session.post(
+        response = self.session.post(
             "http://jwyd.dgut.edu.cn/student/xscj.stuckcj_data.jsp", data=data)
 
         # 解析
-        html = etree.HTML(responsse.text)
-        tr = html.xpath('//table[position() mod 2 = 0]/tbody/tr')
-        score_list = [item.xpath('./td/text()') for item in tr]
-        return score_list
+        if not response.status_code == 200:
+            raise HTTPError(f"[HTTP响应错误] HTTP code {response.status_code}")
+        for td in etree.HTML(response.text).xpath('//table[position() mod 2 = 0]/tbody/tr'):
+            yield td.xpath('./td[position()>1]/text()')
